@@ -642,96 +642,6 @@ using namespace std;
     delete [] sr;     
     delete [] sumsignranks;
   }
-
-
-   void sum_of_diff_sign_select(double *X, int *nk, int *num, double *result)
-  {
-    int n=nk[0]; 
-    int k=nk[1];
-    int nu=num[0]; 
-    int i;
-    int j;
-    int m;  
-    int p=0;
-    double r; 
-    double *u;
-    u = new double[k];
-    double **Y = new double* [k];
-    for (i=0; i<k; i++) Y[i]=new double [k];
-
-    for(i=0;i<k;i++)
-      for(j=0;j<k;j++)
-	Y[i][j]=0.0;
-
-    double **data=prepmat(X,n,k);
-
-    // n-nu first
-    for(i=0;i<(n-nu);i++)
-      for(j=1;j<(nu+1);j++) {
-	//go over the pairs
-	r=0;
-	for(m=0;m<k;m++) {
-	  //compute the difference and its squared norm
-	  u[m] = data[i][m]-data[i+j][m];
-	  r+=(u[m]*u[m]);
-	}
-	r=sqrt(r);
-	for(m=0;m<k;m++)
-	  //compose the sign
-	  u[m]=u[m]/r;
-	for(m=0;m<k;m++)
-	  for(p=0;p<k;p++)
-	    if(p<(m+1)) 
-	      //compute the outer product elements
-	      //and sum to the sum matrix
-	      Y[m][p]+=(u[m]*u[p]);
-      } // j, i
-    
-    // the rest
-    for(i=(n-nu);i<(n-1);i++)
-      for(j=(i+1);j<n;j++) {
-	//go over the pairs
-	r=0;
-	for(m=0;m<k;m++) {
-	  //compute the difference and its squared norm
-	  u[m] = data[i][m]-data[j][m];
-	  r+=(u[m]*u[m]);
-	}
-	r=sqrt(r);
-	for(m=0;m<k;m++)
-	  //compose the sign
-	  u[m]=u[m]/r;
-	for(m=0;m<k;m++)
-	  for(p=0;p<k;p++)
-	    if(p<(m+1)) 
-	      //compute the outer product elements
-	      //and sum to the sum matrix
-	      Y[m][p]+=(u[m]*u[p]);
-      } // j, i
-
-
-    for(m=0;m<(k-1);m++)
-      for(p=(m+1);p<k;p++)
-	//fill up the matrix
-	Y[m][p]=Y[p][m];
-
-    i=0;
-    for(m=0;m<k;m++)
-      for(p=0;p<k;p++) {
-	//put to the result  
-	result[i]=Y[m][p];
-	i++;
-      }
-  
-    for(i=0;i<n;i++)   
-      delete [] data[i]; 
-    delete [] data;
-    delete [] u;
-    for(i=0;i<k;i++)
-      delete [] Y[i];
-    delete [] Y;
-  }
-
    
 
   void symm_huber(double *X, double *V, int *nk, double *cs, double *sigs, double *result)
@@ -815,7 +725,401 @@ using namespace std;
     delete [] Y;
   }
 
+   void symm_huber_inc(double *X, double *V, int *nk, double *cs, double *sigs, int *num, double *result)
+  {
+    int n=nk[0]; 
+    int k=nk[1]; 
+    int nu=num[0];
+    int i;
+    int j;
+    int l;
+    int m;  
+    int p=0;
+    double csq=cs[0];
+    double sigsq=sigs[0];
+    double rd=0.0; 
+    double *u;
+    u = new double[k];
+    double *r;
+    r = new double[k];
+    double w;
+    double **Y = new double* [k];
+    for (i=0; i<k; i++) Y[i]=new double [k];
+ 
+    for(i=0;i<k;i++)
+      for(j=0;j<k;j++)
+    Y[i][j]=0.0;
   
+    for(i=0;i<k;i++)
+       r[i]=0.0;
+ 
+    for(i=0;i<(n-nu);i++) 
+      for(j=(i+1);j<(i+nu+1);j++) {
+    //go over the pairs
+    for(m=0;m<k;m++) {
+      //compute the difference and its squared norm
+      u[m] = X[m*n+i]-X[m*n+j];
+          //compute the Mahalanobis distance
+          for(l=0;l<k;l++) 
+           r[l] += u[m]*V[l*k+m];
+    }
+ 
+ 
+       for(m=0;m<k;m++)
+        rd += r[m]*u[m];
+ 
+       if(rd<=csq){ w=1/sigsq;
+       }else w=(csq/rd)/sigsq;
+  
+       for(m=0;m<k;m++)
+        r[m]=0.0;
+        
+       rd=0.0;
+         
+    for(m=0;m<k;m++){
+      for(p=0;p<k;p++){
+        if(p<(m+1)) {
+          //compute the outer product elements
+          //and sum to the sum matrix
+          Y[m][p]+=w*u[m]*u[p];
+            }
+          }
+        }   
+      } // j, i
+  
+    for(i=(n-nu);i<(n-1);i++) 
+      for(j=(i+1);j<n;j++) {
+    //go over the pairs
+    for(m=0;m<k;m++) {
+      //compute the difference and its squared norm
+      u[m] = X[m*n+i]-X[m*n+j];
+          //compute the Mahalanobis distance
+          for(l=0;l<k;l++) 
+           r[l] += u[m]*V[l*k+m];
+    }
+ 
+ 
+       for(m=0;m<k;m++)
+        rd += r[m]*u[m];
+ 
+       if(rd<=csq){ w=1/sigsq;
+       }else w=(csq/rd)/sigsq;
+  
+       for(m=0;m<k;m++)
+        r[m]=0.0;
+        
+       rd=0.0;
+         
+    for(m=0;m<k;m++){
+      for(p=0;p<k;p++){
+        if(p<(m+1)) {
+          //compute the outer product elements
+          //and sum to the sum matrix
+          Y[m][p]+=w*u[m]*u[p];
+            }
+          }
+        }   
+      } // j, i
+ 
+ 
+    for(m=0;m<(k-1);m++)
+      for(p=(m+1);p<k;p++)
+    //fill up the matrix
+    Y[m][p]=Y[p][m];
+ 
+    i=0;
+    for(m=0;m<k;m++)
+      for(p=0;p<k;p++) {
+    //put to the result  
+    result[i]=Y[m][p];
+    i++;
+      }
+   
+    delete [] r;
+    delete [] u;
+    for(i=0;i<k;i++)
+      delete [] Y[i];
+    delete [] Y;
+  }
+
+   void sum_of_diff_sign_select(double *X, int *nk, int *num, double *result)
+  {
+    int n=nk[0]; 
+    int k=nk[1];
+    int nu=num[0]; 
+    int i;
+    int j;
+    int m;  
+    int p=0;
+    double r; 
+    double *u;
+    u = new double[k];
+    double **Y = new double* [k];
+    for (i=0; i<k; i++) Y[i]=new double [k];
+ 
+    for(i=0;i<k;i++)
+      for(j=0;j<k;j++)
+    Y[i][j]=0.0;
+ 
+    double **data=prepmat(X,n,k);
+ 
+    // n-nu first
+    for(i=0;i<(n-nu);i++)
+      for(j=1;j<(nu+1);j++) {
+    //go over the pairs
+    r=0;
+    for(m=0;m<k;m++) {
+      //compute the difference and its squared norm
+      u[m] = data[i][m]-data[i+j][m];
+      r+=(u[m]*u[m]);
+    }
+    r=sqrt(r);
+    for(m=0;m<k;m++)
+      //compose the sign
+      u[m]=u[m]/r;
+    for(m=0;m<k;m++)
+      for(p=0;p<k;p++)
+        if(p<(m+1)) 
+          //compute the outer product elements
+          //and sum to the sum matrix
+          Y[m][p]+=(u[m]*u[p]);
+      } // j, i
+     
+    // the rest
+    for(i=(n-nu);i<(n-1);i++)
+      for(j=(i+1);j<n;j++) {
+    //go over the pairs
+    r=0;
+    for(m=0;m<k;m++) {
+      //compute the difference and its squared norm
+      u[m] = data[i][m]-data[j][m];
+      r+=(u[m]*u[m]);
+    }
+    r=sqrt(r);
+    for(m=0;m<k;m++)
+      //compose the sign
+      u[m]=u[m]/r;
+    for(m=0;m<k;m++)
+      for(p=0;p<k;p++)
+        if(p<(m+1)) 
+          //compute the outer product elements
+          //and sum to the sum matrix
+          Y[m][p]+=(u[m]*u[p]);
+      } // j, i
+ 
+ 
+    for(m=0;m<(k-1);m++)
+      for(p=(m+1);p<k;p++)
+    //fill up the matrix
+    Y[m][p]=Y[p][m];
+ 
+    i=0;
+    for(m=0;m<k;m++)
+      for(p=0;p<k;p++) {
+    //put to the result  
+    result[i]=Y[m][p];
+    i++;
+      }
+   
+    for(i=0;i<n;i++)   
+      delete [] data[i]; 
+    delete [] data;
+    delete [] u;
+    for(i=0;i<k;i++)
+      delete [] Y[i];
+    delete [] Y;
+  }
+  
+  void symm_mvtmle(double *X, double *V, int *nk, double *df, double *result)
+  {
+    int n=nk[0]; 
+    int k=nk[1]; 
+    int i;
+    int j;
+    int l;
+    int m;  
+    int p=0;
+    double rd=0.0; 
+    double *u;
+    u = new double[k];
+    double *r;
+    r = new double[k];
+    double w;
+    double **Y = new double* [k];
+    for (i=0; i<k; i++) Y[i]=new double [k];
+ 
+    for(i=0;i<k;i++)
+      for(j=0;j<k;j++)
+    Y[i][j]=0.0;
+  
+    for(i=0;i<k;i++)
+       r[i]=0.0;
+ 
+    for(i=0;i<(n-1);i++) 
+      for(j=(i+1);j<n;j++) {
+    //go over the pairs
+    for(m=0;m<k;m++) {
+      //compute the difference and its squared norm
+      u[m] = X[m*n+i]-X[m*n+j];
+          //compute the Mahalanobis distance
+          for(l=0;l<k;l++) 
+           r[l] += u[m]*V[l*k+m];
+    }
+ 
+ 
+       for(m=0;m<k;m++)
+        rd += r[m]*u[m];
+ 
+       w = (df[1]+df[0])/(rd+df[0]);
+  
+       for(m=0;m<k;m++)
+        r[m]=0.0;
+        
+       rd=0.0;
+         
+    for(m=0;m<k;m++){
+      for(p=0;p<k;p++){
+        if(p<(m+1)) {
+          //compute the outer product elements
+          //and sum to the sum matrix
+          Y[m][p]+=w*u[m]*u[p];
+            }
+          }
+        }   
+      } // j, i
+ 
+    for(m=0;m<(k-1);m++)
+      for(p=(m+1);p<k;p++)
+    //fill up the matrix
+    Y[m][p]=Y[p][m];
+ 
+    i=0;
+    for(m=0;m<k;m++)
+      for(p=0;p<k;p++) {
+    //put to the result  
+    result[i]=Y[m][p];
+    i++;
+      }
+   
+    delete [] r;
+    delete [] u;
+    for(i=0;i<k;i++)
+      delete [] Y[i];
+    delete [] Y;
+  }
+
+  void symm_mvtmle_inc(double *X, double *V, int *nk, double *df, int *num, double *result)
+  {
+    int n=nk[0]; 
+    int k=nk[1]; 
+    int nu=num[0];
+    int i;
+    int j;
+    int l;
+    int m;  
+    int p=0;
+    double rd=0.0; 
+    double *u;
+    u = new double[k];
+    double *r;
+    r = new double[k];
+    double w;
+    double **Y = new double* [k];
+    for (i=0; i<k; i++) Y[i]=new double [k];
+ 
+    for(i=0;i<k;i++)
+      for(j=0;j<k;j++)
+    Y[i][j]=0.0;
+  
+    for(i=0;i<k;i++)
+       r[i]=0.0;
+ 
+    for(i=0;i<(n-nu);i++) 
+      for(j=(i+1);j<(i+nu+1);j++) {
+    //go over the pairs
+    for(m=0;m<k;m++) {
+      //compute the difference and its squared norm
+      u[m] = X[m*n+i]-X[m*n+j];
+          //compute the Mahalanobis distance
+          for(l=0;l<k;l++) 
+           r[l] += u[m]*V[l*k+m];
+    }
+ 
+ 
+       for(m=0;m<k;m++)
+        rd += r[m]*u[m];
+ 
+       w = (df[1]+df[0])/(rd+df[0]);
+  
+       for(m=0;m<k;m++)
+        r[m]=0.0;
+        
+       rd=0.0;
+         
+    for(m=0;m<k;m++){
+      for(p=0;p<k;p++){
+        if(p<(m+1)) {
+          //compute the outer product elements
+          //and sum to the sum matrix
+          Y[m][p]+=w*u[m]*u[p];
+            }
+          }
+        }   
+      } // j, i
+  
+    for(i=(n-nu);i<(n-1);i++) 
+      for(j=(i+1);j<n;j++) {
+    //go over the pairs
+    for(m=0;m<k;m++) {
+      //compute the difference and its squared norm
+      u[m] = X[m*n+i]-X[m*n+j];
+          //compute the Mahalanobis distance
+          for(l=0;l<k;l++) 
+           r[l] += u[m]*V[l*k+m];
+    }
+ 
+ 
+       for(m=0;m<k;m++)
+        rd += r[m]*u[m];
+ 
+       w=(df[1]+df[0])/(rd+df[0]);
+  
+       for(m=0;m<k;m++)
+        r[m]=0.0;
+        
+       rd=0.0;
+         
+    for(m=0;m<k;m++){
+      for(p=0;p<k;p++){
+        if(p<(m+1)) {
+          //compute the outer product elements
+          //and sum to the sum matrix
+          Y[m][p]+=w*u[m]*u[p];
+            }
+          }
+        }   
+      } // j, i
+ 
+ 
+    for(m=0;m<(k-1);m++)
+      for(p=(m+1);p<k;p++)
+    //fill up the matrix
+    Y[m][p]=Y[p][m];
+ 
+    i=0;
+    for(m=0;m<k;m++)
+      for(p=0;p<k;p++) {
+    //put to the result  
+    result[i]=Y[m][p];
+    i++;
+      }
+   
+    delete [] r;
+    delete [] u;
+    for(i=0;i<k;i++)
+      delete [] Y[i];
+    delete [] Y;
+  }
 
   void outer(double *u, double *v, int k, double *uvT)
   {
